----------------------------------------------------------------------------------
-- Module Name: InputCounter - Behavioral 
-- Author:	Mike Field (hamster@snap.net.nz)
-- Revision:	1.0
--
-- Count how long each input pulse is. MinPulse is the shortest point see so far
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity InputCounter is
    Port ( Clk 		: in  STD_LOGIC;
           flipped 	: in  STD_LOGIC;
			  Reset 		: in  STD_LOGIC;
           MinPulse  : out STD_LOGIC_VECTOR (7 downto 0);
           Overflow 	: out STD_LOGIC
			 );
end InputCounter;

architecture Behavioral of InputCounter is
	type reg is record
		pulseLen 	: std_logic_vector(7 downto 0);
		minPulse 	: std_logic_vector(7 downto 0);
		overflow 	: STD_LOGIC;
		valid			: STD_LOGIC;
	end record;
	
	signal r : reg := (x"00", x"FF",'0','0');
	signal n : reg;
begin
	-- Set the outputs
	minPulse <= r.minPulse;
	overflow	<= r.Overflow;
	
	process(flipped, r)
	begin
		n <= r;						-- Default is to preserve the contents
		
		if flipped = '0' then	
			-- what we do when the pulse matches that at previous clock
			if r.pulselen = "11111111" then
				n.overflow <= '1';
			else
				n.pulselen <= r.pulselen + 1;
			end if;
		else
			-- when the signal has flipped 
			if r.valid = '1' and r.minpulse > r.pulselen then
				n.minpulse 	<= r.pulselen;
			end if;
			n.pulseLen  	<= "00000001";
			n.valid <= '1';
		end if;
	end process;
	
	process(clk, reset)
	begin
		if reset = '1' then
			r.pulselen 		<= (others => '0');
			r.minpulse 		<= (others => '1');
			r.overflow		<= '0';
			r.valid			<= '0';
		elsif clk'event and clk = '1' then 
			r <= n;
		end if;
	end process;
end Behavioral;

