----------------------------------------------------------------------------------
-- Module Name: frameCapture - Behavioral 
-- Author:	Mike Field (hamster@snap.net.nz)
-- Revision:	1.0
--
-- Capture the S/PDIF frame.
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity frameCapture is
    Port ( clk 		 : in  STD_LOGIC;
           bitstream	 : in  STD_LOGIC;
           takeSample : in  STD_LOGIC;
           data 		 : out STD_LOGIC_VECTOR (23 downto 0);
           channelA   : out STD_LOGIC;
			  dataValid  : out std_logic);
end frameCapture;

architecture Behavioral of frameCapture is

	signal frame : STD_LOGIC_VECTOR (63 downto 0) := x"0000000000000000";
begin

	process(frame)
	begin
		dataValid <= '0';
		channelA  <= '0';
		if frame(63 downto 56) = "00010111" or
			frame(63 downto 56) = "11101000" then
			dataValid <= '1';
			channelA  <= '1';
		end if;

		if frame(63 downto 56) = "00011101" or
			frame(63 downto 56) = "11100010" then
			dataValid <= '1';
			channelA  <= '1';
		end if;

		if frame(63 downto 56) = "00011011" or
			frame(63 downto 56) = "11100100" then
			dataValid <= '1';
			channelA  <= '0';
		end if;
	end process;

	data( 0) <= not frame(55) xor frame(54);
	data( 1) <= not frame(53) xor frame(52);
	data( 2) <= not frame(51) xor frame(50);
	data( 3) <= not frame(49) xor frame(48);
	data( 4) <= not frame(47) xor frame(46);
	data( 5) <= not frame(45) xor frame(44);
	data( 6) <= not frame(43) xor frame(42);
	data( 7) <= not frame(41) xor frame(40);
	data( 8) <= not frame(39) xor frame(38);
	data( 9) <= not frame(37) xor frame(36);
	data(10) <= not frame(35) xor frame(34);
	data(11) <= not frame(33) xor frame(32);
	data(12) <= not frame(31) xor frame(30);
	data(13) <= not frame(29) xor frame(28);
	data(14)  <= not frame(27) xor frame(26);
	data(15)  <= not frame(25) xor frame(24);
	data(16)  <= not frame(23) xor frame(22);
	data(17)  <= not frame(21) xor frame(20);
	data(18)  <= not frame(19) xor frame(18);
	data(19)  <= not frame(17) xor frame(16);
	data(20)  <= not frame(15) xor frame(14);
	data(21)  <= not frame(13) xor frame(12);
	data(22)  <= not frame(11) xor frame(10);
	data(23)  <= not frame( 9) xor frame( 8);

	process(clk,bitstream)
	begin
		if clk'event and clk='1' and takeSample = '1' then
			frame <= frame(62 downto 0) & bitstream;
		end if;
	end process;

end Behavioral;

